def args_and_imports(be, a, very, poorly, formatted, function, that, noone, should, ever, use, it, has, way, too, many, arguments):
    from pykern.pkdebug import pkdp
    x ='abcdefg'



    def foo():
        return 1
    def bar(): return 3
