"""Front-end command line for :mod:`{{ name }}`.

See :mod:`pykern.pkcli` for how this module is used.

{{ copyright_license_rst }}
"""
import pykern.pkcli
import sys


def main():
    return pykern.pkcli.main("{{ name }}")


if __name__ == "__main__":
    sys.exit(main())
