# -*- coding: utf-8 -*-
"""API wrapper

:copyright: Copyright (c) 2022 RadiaSoft LLC.  All Rights Reserved.
:license: http://www.apache.org/licenses/LICENSE-2.0.html
"""
from pykern.pkcollections import PKDict
from pykern.pkdebug import pkdc, pkdlog, pkdp


class Args:
    # qargs
    pass


class API(PKDict):
    # qcall
    pass


class Result:
    # qres
    pass


class Spec:
    # qspec
    pass
