""":mod:`{{ name }}` package

{{ copyright_license_rst }}
"""
import pkg_resources

try:
    # We only have a version once the package is installed.
    __version__ = pkg_resources.get_distribution("{{ name }}").version
except pkg_resources.DistributionNotFound:
    pass
