### {{ name }}

{{ description }}

Learn more at {{ url }}.

Documentation: https://{{ name }}.readthedocs.io

#### License

License: {{ license }}

Copyright (c) {{ year }} {{ author }}.  All Rights Reserved.
